
from cryptodataaccess import helpers
from cryptodataaccess.Notifications.NotificationsStore import NotificationsStore
from cryptomodel.sent_notification import sent_notification

DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'


class NotificationsMongoStore(NotificationsStore):

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def fetch_unsent_notifications(self):
        raise NotImplementedError()

    def insert_sent_notification(self, comp_not):
        return helpers.server_time_out_wrapper(self, self.do_insert_sent_notification, comp_not)

    def do_insert_sent_notification(self, _sent_notification):
        helpers.do_local_connect(self.configuration)
        sn = sent_notification()
        sn.computed_date = _sent_notification.computed_date
        sn.result = _sent_notification.result
        sn.user_id = _sent_notification.user_id
        sn.user_name = _sent_notification.user_name
        sn.user_email = _sent_notification.user_email
        sn.notification_type = _sent_notification.notification_type

        sn.check_every = _sent_notification.check_every
        sn.start_date = _sent_notification.start_date
        sn.end_date = _sent_notification.end_date
        sn.check_every = _sent_notification.check_every
        sn.is_active = _sent_notification.is_active
        sn.channel_type = _sent_notification.channel_type
        sn.threshold_value = _sent_notification.threshold_value
        sn.source_id = _sent_notification.source_id
        sn.operation = _sent_notification.operation
        sn.computed_date = _sent_notification.computed_date
        sn.result = _sent_notification.result
        sn.save()
        return sent_notification.objects(id=sn.id).first()




from abc import ABCMeta, abstractmethod


class CalculatorStore(metaclass=ABCMeta):

    @abstractmethod
    def insert_computed_notification(self):
        pass

    @abstractmethod
    def fetch_computed_notification_before_date(self):
        pass


from cryptomodel.computed_notification import computed_notification
from cryptomodel.operations import OPERATIONS

from cryptodataaccess.Memory import Memory, CALCULATOR_MEMORY_KEY
from cryptodataaccess.Repository import Repository

DATE_FORMAT = "%Y-%m-%d"


class CalculatorRepository(Repository):

    def __init__(self, calculator_store):
        self.calculator_store = calculator_store
        self.computed_notifications = []
        super(CalculatorRepository, self).__init__()
        memory = Memory(on_add=self.calculator_store.insert_computed_notification,
                        on_edit=None,
                        on_remove=None,
                        items=self.computed_notifications
                        )
        self.memories[CALCULATOR_MEMORY_KEY] = memory

    def add_computed_notification(self, user_id, user_name, user_email, notification_type, check_every, is_active,
                                  start_date, end_date, channel_type, threshold_value, source_id, computed_date, result,
                                  ):
        n = computed_notification(
            user_id=user_id, user_name=user_name, user_email=user_email,
            notification_type=notification_type,
            check_every=check_every,
            start_date=start_date,
            end_date=end_date,
            is_active=is_active,
            channel_type=channel_type, threshold_value=threshold_value, source_id=source_id,
            operation=OPERATIONS.ADDED.name, computed_date=computed_date, result=result)

        self.memories[CALCULATOR_MEMORY_KEY].items.append(n)
        return n

    def edit_computed_notification(self, user_id, user_name, user_email, notification_type, check_every, is_active,
                                  start_date, end_date, channel_type, threshold_value, source_id, computed_date, result):
        n = computed_notification(
            user_id=user_id, user_name=user_name, user_email=user_email,
            notification_type=notification_type,
            check_every=check_every,
            start_date=start_date,
            end_date=end_date,
            is_active=is_active,
            channel_type=channel_type, threshold_value=threshold_value, source_id=source_id,
            operation=OPERATIONS.MODIFIED.name, computed_date=computed_date, result=result)

        self.memories[CALCULATOR_MEMORY_KEY].items.append(n)
        return n

    def get_computed_notification_before_date(self, user_id, date):
        return self.calculator_store.fetch_computed_notifications_before_date(user_id, date)




from cryptodataaccess import helpers
from cryptodataaccess.Calculator.CalculatorStore import CalculatorStore
from cryptomodel.computed_notification import computed_notification, Q

DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'


class CalculatorMongoStore(CalculatorStore):

    def __init__(self, config, log_error):
        self.configuration = config
        self.log_error = log_error

    def fetch_computed_notification_before_date(self, user_id, date):
        return self.transaction_store.do_fetch_computed_notification_before_date(user_id, date)

    def insert_computed_notification(self, comp_not):
        return helpers.server_time_out_wrapper(self, self.do_insert_computed_notification, comp_not)

    def do_insert_computed_notification(self, _computed_notification):
        helpers.do_local_connect(self.configuration)
        cn = computed_notification()
        cn.computed_date = _computed_notification.computed_date
        cn.result = _computed_notification.result
        cn.user_id = _computed_notification.user_id
        cn.user_name = _computed_notification.user_name
        cn.user_email = _computed_notification.user_email
        cn.notification_type = _computed_notification.notification_type

        cn.check_every = _computed_notification.check_every
        cn.start_date = _computed_notification.start_date
        cn.end_date = _computed_notification.end_date

        cn.is_active = _computed_notification.is_active
        cn.channel_type = _computed_notification.channel_type
        cn.threshold_value = _computed_notification.threshold_value
        cn.source_id = _computed_notification.source_id
        cn.operation = _computed_notification.operation
        cn.save()
        return computed_notification.objects(id=cn.id).first()

    def do_fetch_computed_notification_before_date(self, user_id, date):
        helpers.do_local_connect(self.configuration)
        return computed_notification.objects(Q(user_id=user_id) &
                                             Q(computed_date__lte=date)
                                             )



from datetime import datetime

import mock
from bson import ObjectId

from cryptomodel.operations import OPERATIONS
from cryptodataaccess.Memory import NOTIFICATIONS_MEMORY_KEY
from cryptodataaccess.config import configure_app
from cryptodataaccess.Notifications.NotificationsRepository import NotificationsRepository
from cryptodataaccess.Notifications.NotificationsMongoStore import NotificationsMongoStore
import pytest
from cryptodataaccess.helpers import do_local_connect
from cryptomodel.sent_notification import sent_notification

@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_insert_sent_notification():
    config = configure_app()
    store = NotificationsMongoStore(config, mock_log)
    repo = NotificationsRepository(store)
    do_local_connect(config)
    sent_notification.objects.all().delete()
    ut = repo.add_sent_notification(user_name="as",user_email="sa",user_id=1,threshold_value=1,notification_type="BALANCE",
                                    check_every="00:00",channel_type="TELEGRAM",is_active="False",
                                    start_date=datetime.now(),end_date=datetime.now(),
                                    source_id=ObjectId('666f6f2d6261722d71757578'), result="ds", computed_date=datetime.now)



    repo.commit()

    ut = repo.memories[NOTIFICATIONS_MEMORY_KEY].items[0]
    assert (ut.user_id == 1)
    assert (ut.operation == OPERATIONS.ADDED.name)
